"""Contains functions useful for fatsorting drives."""

import os
import subprocess
from . import talk


def fatsortAvailable():
    """Return true if fatsort is available and false otherwise.

    Checks if fatsort is available to the user.

    Returns:
        A boolean signaling whether fatsort is available.
    """
    fatCheck = subprocess.Popen(["bash", "-c", "type fatsort"],
                                stdout=subprocess.DEVNULL,
                                stderr=subprocess.DEVNULL)
    exitCode = fatCheck.wait()
    return bool(not exitCode)


def findDeviceLocations(destinationPath, noninteractive=False, verbose=False,
                        quiet=False):
    """Return device and mount locations of a FAT drive.

    Find device and mount locations of the FAT device corresponding to
    the supplied destination path. If these locations can't be found
    automatically, find them interactively. If all of this fails, return
    a 2-tuple of empty strings.

    Args:
        destinationPath: A string containing a path somewhere on the
            mounted device.
        noninteractive: An optional boolean toggling whether to omit
            interactively finding device and mount locations if doing so
            automatically fails.
        verbose: An optional boolean toggling whether to give extra
            output.
        quiet: An optional boolean toggling whether to omit error
            output.

    Returns:
        A 2-tuple containing device location and mount location strings;
        or, if these locations can't be found, a 2-tuple of empty
        strings.
    """
    # Make sure destination is an absolute path
    destination = os.path.abspath(destinationPath)

    # Get list of FAT devices
    bashListCmd = "mount -t vfat | cut -f 1,3 -d ' '"
    deviceListProcess = subprocess.Popen(["bash", "-c", bashListCmd],
                                         stdout=subprocess.PIPE)

    # Read the devices list from Popen
    deviceString = deviceListProcess.communicate()[0].decode('ascii')
    deviceString = deviceString.rstrip()

    # Check if any FAT devices were found
    if deviceString == '':
        # No FAT devices found, return empty string
        return ('', '')

    # Split deviceString so we get a separate string for each device
    deviceList = deviceString.split('\n')

    # For each device, split into device location and mount location.
    # So in deviceListSep, deviceListSep[i][0] gives the device location
    # and deviceListSep[i][1] gives the mount location of the ith device
    deviceListSep = [deviceList[i].split() for i in range(len(deviceList))]

    # Test if destination path matches any mount locations
    for i in range(len(deviceList)):
        deviceLoc = deviceListSep[i][0]
        mountLoc = deviceListSep[i][1]

        if os.path.commonpath((destination, mountLoc)) == mountLoc:
            # Found a match! Return device and mount location
            return (deviceLoc, mountLoc)

    # Something went wrong with the automation: if not set to
    # non-interactive mode, ask user if any of the FAT devices found
    # earlier match the intended destination; otherwise, just return
    # empty strings
    if not noninteractive:
        # Enumerate each device
        deviceListEnum = ["[%d] %s" % (i, deviceList[i-1])
                          for i in range(1, len(deviceList)+1)]

        # Add option to abort
        deviceListEnum.insert(0, "[0] abort!")

        # Prompt user for which device to use
        talk.status("Failed to find device automatically!", verbose)
        print("Mounted FAT devices:", end='\n\n')
        print(*deviceListEnum, sep='\n', end='\n\n')

        ans = int(
                input("Drive to transfer to or abort [0-%d]: "
                      % (len(deviceListEnum)-1))
                 )

        # Return appropriate device and mount strings
        if ans == 0:
            # User selected abort, so return empty strings
            return ('', '')
        elif ans > len(deviceListEnum)-1:
            talk.error("Invalid index", quiet)
            return ('', '')
        else:
            # Return requested device and mount location strings
            return (deviceListSep[ans-1][0], deviceListSep[ans-1][1])
    else:
        # Non-interactive mode is on, just return empty strings
        return ('', '')


def unmount(deviceLocation, verbose=False):
    """Unmount a device and return whether it was successful."""
    noiseLevel = []
    if verbose:
        noiseLevel += ['-v']

    exitCode = subprocess.Popen(['sudo', 'umount', deviceLocation]
                                + noiseLevel).wait()
    return bool(not exitCode)


def fatsort(deviceLocation, quiet=False):
    """fatsort a device and return whether it was successful."""
    noiseLevel = []
    if quiet:
        noiseLevel += ['-q']

    exitCode = subprocess.Popen(['sudo', 'fatsort', deviceLocation]
                                + noiseLevel).wait()
    return bool(not exitCode)



"""Contains a function to rename directories according to instructions.
Current use of this function is for radio shows, but can be anything.
"""

import os
import re
from . import talk
from .config.rename_targets import name_patterns


def rename(targetDirectory, quiet=False):
    """Rename directories according to regex patterns.

    All regex patterns are given in list name_patterns, which lives in
    rename_targets.py. This list is a list of lists. Each list element
    contains (1) identifying regex to determine whether to rename a
    given directory according to the list's instructions; (2) regex to
    group items in the original directory name; (3) a string to insert
    the matched groups into.

    Args:
        targetDirectory: A string containing the path to the directory
            containing the directories to be renamed. See [*] above
        quiet: A boolean toggling whether to supress error output.
    """
    # It's easiest if we move to the target directory, and move back
    # later
    oldCwd = os.getcwd()
    os.chdir(targetDirectory)

    # Test each directory name for a pattern match
    for dir_name in os.listdir():
        for pattern in name_patterns:
            if re.search(pattern[0], dir_name):
                # Directory name matched pattern
                new_name = re.sub(pattern[1], pattern[2], dir_name)

                # Check if directory already exists. If it's empty, just
                # copy into it. If non-empty, skip renaming.
                if os.path.exists(new_name):
                    if not (os.path.isdir(new_name) and os.listdir(new_name) == []):
                        # Directory name already taken! Move onto next
                        # directory.
                        talk.error("Failed to rename %s; %s already exists!"
                                % (dir_name, targetDirectory + "/" + dir_name))
                        break

                try:
                    os.rename(dir_name, new_name)
                except OSError:
                    talk.error("Failed to rename %s" % dir_name, quiet)

                # Move onto next directory, success or not.
                break

    # Clean up: move back to old cwd
    os.chdir(oldCwd)

    return



"""This contains instructions to auto rename directories.  First give a
search regex to identify a directory to rename, then give regex for the
groups to parse in the original name (make sure to match the whole name,
so backreferencing in Python will work properly) then give a new name
that uses the groups matched. See the function re.sub at
https://docs.python.org/3/library/re.html#re-objects.
"""

name_patterns= [

# Above and Beyond - Group Therapy
# example: Above and Beyond - Group Therapy 190 (2016-07-15) (Vyze) -> ABGT_190
[r'(Above and Beyond)(.+)(\(Vyze\))',r'.*\s(\d{3})\s.*',r'ABGT_\1'],

# Armin van Buuren - A State of Trance
# example: Armin van Buuren - A State Of Trance 826 (10.08.2017) SBD Split Tracks -> ASOT_826
[r'(Armin van Buuren)(.+)(SBD Split Tracks)',r'.*\s(\d{3}(?:\.\d)?)\s.*',r'ASOT_\1'],

]



#!/usr/bin/env python3
# coding: utf-8
"""Copy files to a device and fatsort that device.

Run this on the command line like so:

    $ transfat source1 source2 pathOnDrive

or do

    $ transfat -h

to see how to be fancier. Or read the README.md.
"""

from transfat import fatsort
from transfat import rename
from transfat import system
from transfat import talk
from transfat import transfer


def main():
    """The main script of transfat."""
    # Get runtime arguments
    args = system.getRuntimeArguments()

    # Read the configuration file
    talk.status("Reading config file '%s'" % args.config_file, args.verbose)

    cfgSettings = system.getConfigurationSettings(args.config_file,
                                                  args.default,
                                                  args.quiet)
    if not cfgSettings:
        # Failure
        system.abort(1)
    else:
        # Success
        talk.success("'%s' read" % args.config_file, args.verbose)

    # Get root access if we don't have it already, and restart with it
    # if we don't. No need to do this if we're not fatsorting.
    if not args.no_sort:
        talk.status("Checking root access", args.verbose)

        rootAccess = system.requestRootAccess(cfgSettings,
                                              args.non_interactive,
                                              args.verbose)
        if not rootAccess:
            # Failed to run as root
            talk.error("Failed to run as root!", args.quiet)
            system.abort(1)
        else:
            # Success
            talk.success("Running as root", args.verbose)

    # Warn that this will take a bit of time if we're not fatsorting
    if not args.quiet:
        print("This may take a few minutes . . .")

    # Confirm that fatsort is installed
    if not args.no_sort:
        talk.status("Checking if fatsort is available", args.verbose)

        if fatsort.fatsortAvailable():
            # fatsort available
            talk.success("fatsort is available", args.verbose)
        else:
            # fatsort unavailable
            talk.error("fatsort not found!", args.quiet)
            system.abort(1)

    # Find device and mount location corresponding to provided
    # destination
    talk.status("Finding device and mount locations containing '%s'"
                % args.destination, args.verbose)

    # This function returns empty strings if it failed
    devLoc, mntLoc = fatsort.findDeviceLocations(args.destination,
                                                 args.non_interactive,
                                                 args.verbose,
                                                 args.quiet)
    if devLoc == '':
        # Failure
        talk.error("no FAT device found!", args.quiet)
        system.abort(1)
    else:
        # Success, print the devices
        if args.verbose:
            print("Success\n\nFound device and mount locations:"
                  "\ndevice: %s\nmount: %s" % (devLoc, mntLoc),
                  end='\n\n')

    # Transfer files
    if not args.sources:
        # Get source and destination paths
        talk.status("Getting lists of source and destination paths",
                    args.verbose)

        _, fromFiles, toDirs, toFiles = (
            transfer.getCorrespondingPathsLists(args.sources, args.destination,
                                                args.verbose, args.quiet))

        talk.success("Source and destination locations found", args.verbose)

        # Filter out certain file types based on settings in config file
        talk.status("Filtering out unwanted file types", args.verbose)

        transfer.filterOutExtensions(fromFiles, toFiles, cfgSettings,
                                     args.non_interactive)

        talk.success("Filtering complete", args.verbose)

        # Perform necessary audio file conversions
        talk.status("Starting to convert any audio files that need it",
                    args.verbose)

        # Returns a list of temporary files to remove later
        tmpFiles = transfer.convertAudioFiles(fromFiles, toFiles, cfgSettings,
                                              args.non_interactive,
                                              args.verbose, args.quiet)

        talk.success("Conversions finished", args.verbose)

        # Create necessary directories to transfer to
        talk.status("Creating destination directories", args.verbose)

        transfer.createDirectories(toDirs, args.non_interactive, args.verbose,
                                   args.quiet)

        talk.success("Destination directories created", args.verbose)

        # Copy source files to destination
        talk.status("Copying files", args.verbose)

        transfer.copyFiles(fromFiles, toFiles, cfgSettings, args.non_interactive,
                           args.verbose, args.quiet)

        talk.success("Files copied", args.verbose)

        # Delete temporary files
        talk.status("Removing any temp files", args.verbose)

        transfer.deleteFiles(tmpFiles)

        talk.success("temp files removed", args.verbose)

        # Delete source directories if asked we're asked to. Note that
        # deleteSourceSetting - 1 is equivalent to a prompt flag, given
        # the config setting constant definitions.
        deleteSourceSetting = cfgSettings.getint("DeleteSources")
        promptFlag = deleteSourceSetting - 1

        if (deleteSourceSetting
           and not (args.non_interactive and promptFlag)):
            # Remove sources
            talk.status("Removing source files and directories", args.verbose)

            transfer.deletePaths(args.sources, promptFlag, args.verbose,
                                 args.quiet)

            talk.success("source files and directories removed", args.verbose)

    # If renaming directories, do so
    if args.rename or cfgSettings.getint('RenameByDefault'):
        talk.status("Renaming any matching directories", args.verbose)

        rename.rename(mntLoc, args.quiet)

        talk.success("Matching directories renamed", args.verbose)

    # Unmount and fatsort if we're asked to
    if not args.no_sort:
        # Unmount
        talk.status("Unmounting %s" % mntLoc, args.verbose)

        if not fatsort.unmount(devLoc, args.verbose):
            talk.error("Failed to unmount %s!" % mntLoc, args.quiet)
            system.abort(1)
        else:
            talk.success("%s unmounted" % mntLoc, args.verbose)

        # Fatsort
        talk.status("fatsorting %s" % mntLoc, args.quiet)

        if not fatsort.fatsort(devLoc, args.verbose):
            talk.error("Failed to fatsort %s!" % mntLoc, args.quiet)
            system.abort(1)
        else:
            talk.success("%s fatsorted" % mntLoc, args.verbose)

    # Successful run
    talk.success("All done", args.verbose)

    return


if __name__ == '__main__':
    # This runs from the command line
    main()



"""Contains functions used to copy and process (mostly audio) files."""

import os
import shutil
import subprocess
from . import talk
from .config.constants import NO, YES, PROMPT


def getCorrespondingPathsLists(sourcePaths, destinationPath, verbose=False,
                               quiet=False):
    """Return lists of corresponding source and destination paths.

    Generate corresponding lists of paths for source and destination
    files and source and destination directories. The indices of the two
    file lists will correspond to each other, and similarly, the indices
    of the two directory lists will correspond to each other.

    Args:
        sourcePaths: A list of strings containing source paths, which
            can be files or directories.
        destinationPath: A string containing a destination path for
            where the source files/directories should be transfered to.
        verbose: An optional boolean toggling whether to give extra
            output.
        quiet: An optional boolean toggling whether to omit error
            output.

    Returns:
        A 4-tuple containing (sourceDirs, sourceFiles, destinationDirs,
        destinationFiles) where ...

        sourceDirs: A list of strings containing absolute paths to
            source directories.
        sourceFiles: A list of strings containing absolute paths to
            source files.
        destinationDirs: A list of strings containing absolute paths to
            destination directories.
        destinationFiles: A list of strings containing absolute paths to
            destination files.

        and where the indices of sourceDirs and destinationDirs
        correspond to each other, and, similarly, where the indices of
        sourceFiles and destinationFiles correspond to each other.
    """
    # Make sure source and destination paths are absolute paths
    sourcePaths_ = [os.path.abspath(source) for source in sourcePaths]
    destinationPath_ = os.path.abspath(destinationPath)

    # Generate lists of source and destination paths
    sourceDirs = []
    sourceFiles = []
    destinationDirs = []
    destinationFiles = []

    # Go through each source
    for source in sourcePaths_:
        # Get the parent directory of the source so we can generate the
        # destination path
        parent = os.path.dirname(source)
        parentlen = len(parent)

        # Determine whether the source is a file or directory
        if os.path.isfile(source):
            # The source is a file, so add it to the file lists
            sourceFiles += [source]
            destinationFiles += [destinationPath_ + source[parentlen:]]
        elif os.path.isdir(source):
            # The source is a directory, so add itself and everything
            # inside of it to the appropriate lists
            for root, _, files in os.walk(source):
                sourceDirs += [root]
                sourceFiles += [root + '/' + file for file in files]
                destinationDirs += [destinationPath_ + root[parentlen:]]
                destinationFiles += [destinationPath_
                                     + root[parentlen:]
                                     + '/'
                                     + file
                                     for file in files]
        else:
            # The source is neither a file nor directory. Give a
            # warning.
            talk.error("'%s' does not exist!" % source, quiet)
            talk.status("Proceeding anyway", verbose)

    return (sourceDirs, sourceFiles, destinationDirs, destinationFiles)


def filterOutExtensions(sourceFiles, destinationFiles, configsettings,
                        noninteractive=False):
    """Remove indices corresponding to unwanted files from lists.

    Filter out files of unwanted extensions from the list of source
    files and destination files.

    [*] The indices of the source file list and destination file list
    inputs must correspond to each other.

    Args:
        sourceFiles: A list of strings of absolute paths to source
            files. See [*] above.
        destinationFiles: A list of strings of absolute paths to
            destination files. See [*] above.
        configsettings: A dictionary-like 'configparser.SectionProxy'
            object containing configuration settings from config.ini.
        noninteractive: An optional boolean toggling whether to suppress
            prompts to remove files that may have been requested in the
            configuration file config.ini.

    Returns:
        Nothing. The work performed on the file lists is done in place.
    """
    # Load settings from config file
    imageOption = configsettings.getint('RemoveImages')
    logOption = configsettings.getint('RemoveLog')
    cueOption = configsettings.getint('RemoveCue')
    m3uOption = configsettings.getint('RemoveM3U')
    otherOption = configsettings.getint('RemoveOtherFiletypes')

    # Tuples of file extension types we care about
    audioExt = ('.flac', '.alac', '.aac', '.m4a', '.mp4', '.ogg', '.mp3')
    imageExt = ('.jpg', '.jpeg', '.bmp', '.png', '.gif')
    logExt = ('.log',)
    cueExt = ('.cue',)
    m3uExt = ('.m3u',)

    # Pair each file extension with its corresponding config setting
    extensionList = [[imageExt, imageOption],
                     [logExt, logOption],
                     [cueExt, cueOption],
                     [m3uExt, m3uOption]]

    # Gather all of the non-audio file extensions into one tuple
    nonAudioExt = ()
    for ext in extensionList:
        nonAudioExt += ext[0]

    # Initialize a list of indices corresponding to files to remove
    indexList = []

    # Find which files have extensions that we don't want and mark their
    # indices
    for file_ in destinationFiles:
        if file_.lower().endswith(audioExt):
            # This is an audio file; keep this file for sure
            pass
        elif file_.lower().endswith(nonAudioExt):
            # This matches one of the non-audio extensions. Find which
            # extension it is and remove the file from the file list as
            # instructed to by the config settings.
            for ext, removeOption in extensionList:
                if file_.lower().endswith(ext):
                    # Extension matched! Remove the file according to
                    # the config settings, prompting if necessary.

                    if ((removeOption == PROMPT
                        and (noninteractive
                             or talk.prompt("Move '%s'?" % file_)))
                            or removeOption == NO):
                        # Keep the file in the file list
                        break
                    else:
                        # Add index to list of indices to remove
                        indexList += [destinationFiles.index(file_)]
        else:
            # This is some other kind of file. Remove the file according
            # to the config settings, prompting if necessary.
            if ((otherOption == PROMPT
                 and (noninteractive or talk.prompt("Move '%s'?" % file_)))
                    or otherOption == NO):
                # Keep the file in the file list
                pass
            else:
                # Add index to list of indices to remove
                indexList += [destinationFiles.index(file_)]

    # Remove files we don't want from the file lists, going through the
    # indices in reverse order
    for index in indexList[::-1]:
        sourceFiles.pop(index)
        destinationFiles.pop(index)

    return


def createDirectories(directoriesList, noninteractive=False, verbose=False,
                      quiet=False):
    """Create directories specified by a list.

    Create all of the directories specified in a list, asking whether to
    overwrite any files blocking the way as necessary.

    Args:
        directoriesList: A list of strings containing absolute paths to
            directories to be created.
        noninteractive: An optional boolean signaling not to overwrite
            files with directories, and not to prompt for this.
        verbose: An optional boolean toggling whether to give extra
            output.
        quiet: An optional boolean toggling whether to omit error
            output.
    """
    # Determine whether to prompt to overwrite files
    if noninteractive:
        doprompt = False
    else:
        doprompt = True

    # Create each directory
    for targetDir in directoriesList:
        try:
            # Check if the directory already exists; if it does, move on
            # to the next directory.
            talk.status("Checking %s" % targetDir, verbose)

            if os.path.isdir(targetDir):
                # Already a directory
                talk.status("%s already exists" % targetDir, verbose)

                continue

            # Check if we're attempting to overwrite a file
            if os.path.isfile(targetDir):
                # Prompt to overwrite if necessary
                if (doprompt
                   and talk.prompt("%s is a file. Overwrite?" % targetDir)):
                    # Overwrite - remove the file that's in the way
                    os.remove(targetDir)
                else:
                    # Don't overwrite file with directory.
                    talk.error("Attempting to overwrite a file with a"
                               " directory!", quiet)
                    raise OSError("Cannot overwrite a file with a directory!")

            # Create directory
            talk.status("Creating %s" % targetDir, verbose)
            os.makedirs(targetDir)
        except OSError:
            talk.error("Failed to create %s!" % targetDir, quiet)

    return


def convertAudioFiles(sourceFiles, destinationFiles, configsettings,
                      noninteractive=False, verbose=False, quiet=False):
    """Convert non-mp3 audio files to mp3.

    Uses FFmpeg to convert audio files with non-mp3 extensions (as
    specified in the config settings) to mp3s. Returns a list of paths
    to the mp3 files created, and updates the source and destination
    file lists in place, replacing the original files with the newly
    converted files.

    The input arguments for the source and destination files are
    expected to be in terms of absolute paths; [*] furthermore, their
    indices are expected to correspond to each other.

    If the user has an old version of FFmpeg, it's quite possible that
    metadata will fail to transfer to the converted file. On later
    versions this is done by default, so I haven't specified that option
    here.

    Args:
        sourceFiles: A list of strings of absolute paths to source
            files. See [*] above.
        destinationFiles: A list of strings of absolute paths to
            destination files. See [*] above.
        configsettings: A dictionary-like 'configparser.SectionProxy'
            object containing configuration settings from config.ini.
        noninteractive: An optional boolean signalling to never ask to
            convert files that it would otherwise prompt for, and
            furthermore, to not do such conversions.
        verbose: An optional boolean toggling whether to give extra
            output.
        quiet: An optional boolean toggling whether to omit both error
            output and output to signal that the non-interactive flag
            has prevented a conversion from taking place.

    Returns:
        A list of strings containing the absolute paths of the files
        created by conversion. Also modifies the source and destination
        file lists in place such that the original files are replaced by
        the newly converted files.
    """
    # Quality setting for conversions. See:
    # https://trac.ffmpeg.org/wiki/Encode/MP3
    QUALITY = '0'

    # Load extensions to convert from config file
    flacConvert = configsettings.getint('ConvertFLACtoMP3')
    alacConvert = configsettings.getint('ConvertALACtoMP3')
    aacConvert = configsettings.getint('ConvertAACtoMP3')
    m4aConvert = configsettings.getint('ConvertM4AtoMP3')
    mp4Convert = configsettings.getint('ConvertMP4toMP3')
    oggConvert = configsettings.getint('ConvertOGGtoMP3')

    # Put these extensions in a list along with the option specifying
    # whether to prompt. Given that PROMPT is 2, YES is 1, and NO is 0,
    # we have that promptOption = convertOption - 1
    extensionList = []

    if flacConvert:
        extensionList += [['.flac', flacConvert - 1]]
    if alacConvert:
        extensionList += [['.alac', alacConvert - 1]]
    if aacConvert:
        extensionList += [['.aac', aacConvert - 1]]
    if m4aConvert:
        extensionList += [['.m4a', m4aConvert - 1]]
    if mp4Convert:
        extensionList += [['.mp4', mp4Convert - 1]]
    if oggConvert:
        extensionList += [['.ogg', oggConvert - 1]]

    # Make sure we don't prompt if we're in non-interactive mode
    if noninteractive:
        for pair in extensionList:
            if pair[1] == PROMPT:
                # Don't convert this extension
                pairIndex = extensionList.index(pair)
                extensionList.pop(pairIndex)

    # Return an empty list if we don't need to convert anything
    if not extensionList:
        return []

    # We need to look for files to convert. Determine how noisy FFmpeg
    # should be.
    if quiet:
        logsetting = 'fatal'
    elif verbose:
        logsetting = 'info'
    else:
        logsetting = 'warning'

    # List of files converted
    convertedFiles = []

    # Don't prompt more than once to convert the same file extension in
    # the same directory.  Initialize a whitelist and blacklist for
    # this, [**] which will contain lists of two-tuples of ("dirpath",
    # "extension")
    whitelist = []
    blacklist = []

    # Convert each file as necessary
    for oldFile in sourceFiles:
        for extension, prompt in extensionList:
            # Find if the extensions match
            extensionMatch = oldFile.lower().endswith(extension)

            if extensionMatch:
                # An extension matched!
                if prompt:
                    # Work out whether we're on the whitelist,
                    # blacklist, or whether we should prompt for this
                    # file. See [**] above for more details.
                    container = os.path.dirname(oldFile)

                    if (container, extension) in whitelist:
                        # Convert the file
                        pass
                    elif (container, extension) in blacklist:
                        # Move on to next file
                        break
                    else:
                        # Prompt and modify white/black-lists
                        # accordingly
                        if talk.prompt(
                           "Convert %s and other %s's in the same directory?"
                           % (oldFile, extension)):
                            # Add to whitelist and convert
                            whitelist += [(container, extension)]
                        else:
                            # Add to blacklist and move on to next file
                            blacklist += [(container, extension)]
                            break

                # Convert the file!
                talk.status("Converting %s" % oldFile, verbose)

                newFile = oldFile[:-len(extension)] + '.mp3'
                command = (['ffmpeg']
                           + ['-n']
                           + ['-hide_banner']
                           + ['-loglevel', logsetting]
                           + ['-i', oldFile]
                           + ['-codec:a', 'libmp3lame']
                           + ['-qscale:a', QUALITY]
                           + [newFile])

                # Give stdin and stdout to user and wait for completion
                convertProcess = subprocess.Popen(command)
                exitCode = convertProcess.wait()

                if exitCode:
                    # Failed to convert
                    talk.error("Failed to convert %s" % oldFile, quiet)
                else:
                    # Success. Add to list of converted files
                    convertedFiles += [newFile]

                    # Swap the source and destination files with the new
                    # converted file-name.
                    oldFileIndex = sourceFiles.index(oldFile)
                    oldDestination = destinationFiles[oldFileIndex]
                    newDestination = oldDestination[:-len(extension)] + '.mp3'

                    sourceFiles[oldFileIndex] = newFile
                    destinationFiles[oldFileIndex] = newDestination

                # Move on to next file
                break

    return convertedFiles


def copyFiles(sourceFiles, destinationFiles, configsettings,
              noninteractive=False, verbose=False, quiet=False):
    """Copy files from a source to a destination.

    Use cp with options specified in config settings to copy each source
    file into a destination file.

    [*] The indices of the source file list and destination file list
    inputs must correspond to each other.

    Args:
        sourceFiles: A list of strings of absolute paths to source
            files. See [*] above.
        destinationFiles: A list of strings of absolute paths to
            destination files. See [*] above.
        configsettings: A dictionary-like 'configparser.SectionProxy'
            object containing configuration settings from config.ini.
        noninteractive: An optional boolean signalling to never run cp
            with its interactive flag.
        verbose: An optional boolean toggling whether to run cp with its
            verbose flag.
        quiet: An optional boolean toggling whether to omit error
            output.
    """
    # Initialize list of options to run cp with
    cpOptions = []

    # Determine whether to overwrite destination files if there's a
    # conflict
    overwritesetting = configsettings.getint('OverwriteDestinationFiles')

    if overwritesetting == YES:
        # cp --force
        cpOptions += ['-f']
    elif overwritesetting == PROMPT and not noninteractive:
        # cp --interactive
        cpOptions += ['-i']
    else:
        # cp --no-clobber
        cpOptions += ['-n']

    # Determine whether to be verbose
    if verbose:
        cpOptions += ['-v']

    # Copy the files to the destination directory
    for source, destination in zip(sourceFiles, destinationFiles):
        # Give stdin and stdout to user and wait for completion
        copyProcess = subprocess.Popen(["cp", source, destination] + cpOptions)
        exitCode = copyProcess.wait()

        if exitCode:
            # Failed to copy
            talk.error("Failed to copy %s" % source, quiet)

    return


def deletePaths(paths, doprompt=True, verbose=False, quiet=False):
    """Delete a list of files and directories possibly containing files.

    Removes the files and directories (recursively) specified, prompting
    if necessary.

    Args:
        paths: A list of strings containing absolute paths.
        doprompt: An optional boolean signalling to prompt before
            deleting anything.
        verbose: An optional boolean toggling whether to give extra
            output.
        quiet: An optional boolean toggling whether to omit error
            output.
    """
    for thing in paths:
        # Prompt to delete if necessary
        if doprompt:
            if not talk.prompt("Remove %s?" % thing):
                # Don't delete this thing
                break

        talk.status("Removing %s" % thing, verbose)

        # Delete the thing!
        try:
            if os.path.isfile(thing):
                os.remove(thing)
            elif os.path.isdir(thing):
                shutil.rmtree(thing)
            elif not os.path.exists(thing):
                # Nothing here
                raise OSError
            else:
                # Something very strange happened
                raise UserWarning
        except (OSError, shutil.Error):
            # Such error!
            talk.error("Failed to remove %s" % thing, quiet)

    return


def deleteFiles(filePaths, quiet=False):
    """Delete a list of files."""
    for path in filePaths:
        try:
            os.remove(path)
        except OSError:
            talk.error("Failed to remove %s!" % path, quiet)
    return



"""Contains functions related to interfacing with the parent shell.

Most of these functions are specific to transfat, but a few aren't. The
ones that are specific will be marked so.
"""

import argparse
import configparser
import os
import subprocess
import sys
import transfat.config.constants
from . import talk
from .version import NAME, VERSION


def getRuntimeArguments():
    """Return command line arguments as attributes of an object.

    Specific to running transfat.

    Returns:
        An object of type 'argparse.Namespace' containing the runtime
        arguments as attributes. See argparse documentation for more
        details.
    """
    CONFIGPATH = getConfigurationFilePath()

    parser = argparse.ArgumentParser(
            prog=NAME,
            description="%(prog)s"
                        " - transfer audio files to a FAT device"
                        " and sort into natural order")
    parser.add_argument(
            "sources",
            nargs='*',
            type=str,
            help="path to source directories or files")
    parser.add_argument(
            "destination",
            type=str,
            help="path to destination directory or file")
    parser.add_argument(
            "--config-file",
            help="use specified config file",
            type=str,
            default=CONFIGPATH)
    parser.add_argument(
            "--default",
            help="use default settings from config file",
            action="store_true")
    parser.add_argument(
            "--find-config",
            nargs=0,
            help="show location of config file and exit",
            action=ConfigPrintAction)
    parser.add_argument(
            "--no-sort",
            help="do not unmount and fatsort",
            action="store_true")
    parser.add_argument(
            "--rename",
            help="rename name-pattern matched directories",
            action="store_true")
    parser.add_argument(
            "--version",
            action='version',
            version="%(prog)s " + VERSION)
    noiseoptions = parser.add_mutually_exclusive_group()
    noiseoptions.add_argument(
            "--verbose",
            help="give maximal output",
            action="store_true")
    noiseoptions.add_argument(
            "--quiet", "--silent",
            help="give minimal output",
            action="store_true")
    parser.add_argument(
            "-n", "--non-interactive",
            help="never prompt user for input",
            action="store_true")

    arguments = parser.parse_args()

    return arguments


def getConfigurationFilePath():
    """Return a string containing the path of the configuration file."""
    return os.path.dirname(transfat.config.constants.__file__) + '/config.ini'


class ConfigPrintAction(argparse.Action):
    """Custom argparse action to print location of configuration file."""
    def __init__(self, option_strings, *args, **kwargs):
        super(ConfigPrintAction, self).__init__(option_strings, *args, **kwargs)
    def __call__(self, parser, namespace, values, option_string=None):
        print("config file located at %s" % getConfigurationFilePath())
        sys.exit(0)


def getConfigurationSettings(configPath, default=False, quiet=False):
    """Read settings from a config file and return settings.

    Reads configuration settings from a file with the configparser
    module, choosing between three sections of settings depending on the
    flags given when calling this function. Specific to transfat.

    Args:
        configPath: A string containing the path to the configuration
            file.
        default: An optional boolean toggling whether to read the
            'default' section of the config file.
        quiet: An optional boolean toggling whether to omit error
            output.

    Returns:
        A dictionary-like 'configparser.sectionproxy' object containing
        the settings loaded from the configuration file in the case of
        sucess; otherwise returns None.
    """
    # Instantiate the parser
    config = configparser.ConfigParser()

    # If the method read is unsuccessful it returns an empty list.
    if config.read(configPath) == []:
        # No good!
        talk.error("'%s' is not a valid configuration file!" % configPath,
                   quiet)
        return None
    else:
        # Read successful. Select which section of config file to use
        if default:
            # Use default section of config file
            configDict = config['DEFAULT']
        else:
            # Use user section of config file
            configDict = config['user']

        return configDict


def requestRootAccess(configsettings, noninteractive=False, verbose=False):
    """Ensure script is running as root.

    Return true if we're running as root, or false if we can't get root;
    otherwise, obtain root credentials, terminate the program, and
    restart as root.

    Args:
        configsettings: A dictionary-like 'configparser.SectionProxy'
            object containing configuration settings from config.ini.
        noninteractive: An optional boolean toggling whether to ask for
            root if not already a root process.
        verbose: An optional boolean toggling whether to give extra
            output.

    Returns:
        A boolean signaling whether we are root. Another common exit
        from this function is through terminating the program and
        restarting as root.
    """
    # Check if we're already running as root
    euid = os.geteuid()

    if euid == 0:
        # Already running as root
        return True

    # Check if we have root passphrase cached already; exit code of the
    # Popen command will be non-zero if we don't have credentials, and
    # will be zero if we do
    rootCheck = subprocess.Popen(["sudo", "-n", "echo"],
                                 stdout=subprocess.DEVNULL,
                                 stderr=subprocess.DEVNULL)
    exitCode = rootCheck.wait()

    # If we're running non-interactively and we don't have access to
    # root credentials, return false
    if noninteractive and exitCode:
        return False

    # Assume we cache credentials by default (i.e., we run 'sudo'
    # instead of 'sudo -k'); change this below if needed
    cacheOption = []

    # If we don't already have access to root credentials, determine
    # whether to cache root credentials when we ask for them
    if exitCode:
        # Get config settings for caching root credentials
        cache = configsettings.getint('UpdateUserCredentials')

        # Prompt whether to cache root credentials if necessary
        if cache == transfat.config.constants.PROMPT:
            # and store the answer in cache
            cache = talk.prompt("Remember root access passphrase?")

        # Run 'sudo -k' if we aren't caching credentials
        if cache == transfat.config.constants.NO:
            cacheOption = ['-k']

    # Replace currently-running process with root-access process
    talk.status("Prompting for passphrase to restart as root", verbose)

    sudoCmd = (['sudo']
               + cacheOption
               + [sys.executable]
               + sys.argv
               + [os.environ])
    os.execlpe('sudo', *sudoCmd)


def abort(code):
    """Exit program with an exit code."""
    talk.aborting()
    sys.exit(code)



from setuptools import setup
from transfat.version import NAME, VERSION

setup(
    name=NAME,
    version=VERSION,
    description='Play audio files on your car stereo and maintain sanity',
    url='https://github.com/mwiens91/trans-fat',
    author='Matt Wiens',
    author_email='mwiens91@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3 :: Only',
    ],
    data_files=[('/usr/local/man/man1', ['extras/transfat.1']),],
    packages=['transfat', 'transfat.config'],
    package_data={'transfat.config': ['config.ini']},
    entry_points={
        'console_scripts': ['transfat = transfat.main:main'],
    },
)



"""Contains functions for communicating with a user."""

import distutils.util
import sys
from .version import NAME


def prompt(query):
    """Prompt a yes/no question and get an answer.

    A simple function to ask yes/no questions on the command line.
    Credit goes to Matt Stevenson. See:
    http://mattoc.com/python-yes-no-prompt-cli.html

    Args:
        query: A string containing a question.

    Returns:
        A boolean corresponding to the answer to the question asked.
    """
    sys.stdout.write("%s [y/n]: " % query)
    val = input().lower()
    try:
        result = distutils.util.strtobool(val)
    except ValueError:
        # Result no good! Ask again.
        sys.stdout.write("Please answer with y/n\n")
        return prompt(query)
    return result


def status(message, verbose=True):
    """Print a status update if a flag is true."""
    if verbose:
        print(message)
    return


def success(message, verbose=True):
    """Print a success message if a flag is true."""
    if verbose:
        print("Success: " + message)
    return


def error(error_message, quiet=False):
    """Print an error message to stderr if a flag is false."""
    if not quiet:
        print("ERROR: " + error_message, file=sys.stderr)
    return


def aborting():
    """Prints that the program is aborting."""
    print("Aborting %s" % NAME)
    return


if __name__ == '__main__':
    # Self-test code
    error("oh mah %s!" % "god")
    aborting()



"""Contains constants used in the 'config.ini' file.

Much of the logic that uses these constants relies very much on the
precise values these constants take. If you want to change these values,
you should look at every occurance of their use to ensure that things
won't break.
"""

NO = 0
YES = 1
PROMPT = 2
